/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:		 Anpassungen an AVR-NET-IO von Shortie
 known Problems: none
 Version:        12.01.2010
 Description:    Webserver Config-File

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#ifndef _CONFIG_H_
	#define _CONFIG_H_
	
	//-----------------------------------------------------------------------
	// MAC Adresse des Webservers
	//-----------------------------------------------------------------------
	
	#define MYMAC1	0x00	// Dies ist die Default MAC-Adresse von Ulrich
	#define MYMAC2	0x20	// Radig und sollte durch eigene Werte ersetzt
	#define MYMAC3	0x18	// ersetzt werden.
	#define MYMAC4	0xB1	
	#define MYMAC5	0x15	// Pollin nutzt fr das AVR-NET-IO Board eigene
	#define MYMAC6	0x6F	// Adresse die dem Aufkleber zu entnehmen ist.
	
	// Bei Nutzung des TFTP Bootloaders sollten diese mit den Einstellungen
	// dort bereinstimmen.
	
	#define USERDEFINED_MAC	1	// damit kann MAC ber Console eingestellt werden
	
	// Bei USERDEFINED_MAC 1 wird auch vom Bootloader nach erstmaliger 
	// Einstellung die eingestellte MAC-Adresse verwendet.

	//-----------------------------------------------------------------------
	// Pollin AVR-NET-IO Einstellung (siehe auch Ende der Datei)
	//-----------------------------------------------------------------------

	#define USE_NETIO		1	// 1 = Pollin AVR-NET-IO / 0 = Radig ETH_M32_EX
	#define USE_NETIO_32	1	// 1 = AVR-NET-IO mit ATmega32 / 0 = ATmega644(p)

	//-----------------------------------------------------------------------
	// AD-Wandler benutzen
	//-----------------------------------------------------------------------
	
	#define USE_ADC			1
	#define	ADC_START		0	// ab welchem Kanal (0-7) der ADC genutzt wird

	//-----------------------------------------------------------------------
	
	//ETH_M32_EX (www.ulrichradig.de)
	#define USE_ENC28J60	1
	//Holger Buss (www.mikrocontroller.com) Mega32-Board
	#define USE_RTL8019		0
	
	//Konfiguration der PORTS (HEX)
	//1=OUTPUT / 0=INPUT
	#define OUTA 		0x07
	#define OUTC 		0x06
	#define OUTD 		0x00
	//Achtung!!!! an PORTB ist der ENC
	//nur ndern wenn man wei was man macht!

    //Watchdog timer for the ENC2860, resets the stack if timeout occurs
    #define WTT 1200 //Watchdog timer in timer interrupt

	//Umrechnung von IP zu unsigned long
	#define IP(a,b,c,d) ((unsigned long)(d)<<24)+((unsigned long)(c)<<16)+((unsigned long)(b)<<8)+a

	//Taktfrequenz
	//#define F_CPU 20000000UL
    #define F_CPU 16000000UL	
	//#define F_CPU 14745600UL
	//#define F_CPU 11059200UL
	
	//Timertakt intern oder extern
	#define EXTCLOCK 0 // 0=Intern 1=Externer Uhrenquarz

	//Baudrate der seriellen Schnittstelle
	#define BAUDRATE 9600
	
	//-----------------------------------------------------------------------
	// Variablen Array - wird bei USE_NETIO 1 in Zeile 381 vergrert
	//-----------------------------------------------------------------------

	#define MAX_VAR_ARRAY	10
	
	//-----------------------------------------------------------------------
	// Timeanpassung aus Wil-Code
	//-----------------------------------------------------------------------

	// Zeitbasis fr Timerinterrupt ist 25 ms (wird nur in messung.c benutzt)
	#define TIMERBASE	25

	//-----------------------------------------------------------------------
	// Netzwerkeinstellungen und Netzwerkdienste
	//-----------------------------------------------------------------------

	#define MYIP			IP(192,168,0,99)	// IP des Webservers *
    #define ROUTER_IP		IP(192,168,0,1)		// IP des Routers *
	#define NETMASK			IP(255,255,255,0)	// Netzwerkmaske *
	
	// *) Einstellungen bei USE_NETIO 1 weiter unten machen

	//-----------------------------------------------------------------------
	// HTTP-Server
	//-----------------------------------------------------------------------
	
	#define USE_HTTPD			1	// Webserver
	#define HTTP_REFRESH		1	// Refresh der Webseite alle 60 Sekunden
	#define	HTTP_CSVDATEN		1	// Ausgabe der Daten als CSV
	#define	HTTP_STATUSPAGE		1	// Ausgabe der Statuspage (0 spart Platz)
	#define HTTP_AUTH_DEFAULT	1	// Webserver mit Passwort? (0 = mit Passwort)
	#define HTTP_AUTH_STRING	"admin:uli1"	// AUTH String "USERNAME:PASSWORT"
												// max 14Zeichen
	
	//-----------------------------------------------------------------------
	// TELNET-Server			Anfnger lassen es bitte beides auf 0
	//-----------------------------------------------------------------------
	
	#define USE_TELNETD		0	// Telnetserver fr RS232 oder CMD_TELNET
    #define CMD_TELNET      0	// Commandos und Ausgaben erfolgen ber Telnet

	//-----------------------------------------------------------------------
	// Zustzliche Netzwerkkommandos
	//-----------------------------------------------------------------------
	
	#define USE_NETCMDS		1	// Schaltet PING, ARP, TCP und VER an
	
 	//-----------------------------------------------------------------------
	// DHCP-Server einbinden
	//-----------------------------------------------------------------------
	
    #define USE_DHCP		1	// 1 = DHCP Client on
  
	//-----------------------------------------------------------------------
    // DNS-Server einbinden - IP kann von DHCP berschrieben werden
	//-----------------------------------------------------------------------
	
    #define USE_DNS			0	// 1 = DNS Client on
    #define DNS_IP			IP(192,168,0,254)
    
	//-----------------------------------------------------------------------
	// NTP-Server einbinden - IP kann von DHCP berschrieben werden
	//-----------------------------------------------------------------------
	
	#define USE_NTP			1		// 1 = NTP Client on
	#define NTP_REFRESH 	1140	// NTP refresh before stack timeout occurs...
	#define NTP_IP			IP(77,37,6,59)
	
	//-----------------------------------------------------------------------
	// SYSLOG-Server einbinden - IP kann im EEPROM berschrieben werden
	//-----------------------------------------------------------------------
	
	#define USE_SYSLOG		1 
	#define SYSLOG_IP		IP(192,168,0,10)

	//-----------------------------------------------------------------------
	// Wake on Lan einbinden
	//-----------------------------------------------------------------------

	#define USE_WOL			0	// 1 = WOL on
	#define WOL_BCAST_IP	IP(192,168,0,255)				// Broadcast fr WOL
	#define WOL_MAC 		{0x00,0x1A,0xA0,0x9C,0xC6,0x0A}	// MAC die geweckt wird

	//-----------------------------------------------------------------------
	// Pollin ADD-ON fr AVR-NET-IO-Board (nur IR implementiert bislang)
	//-----------------------------------------------------------------------

	#define USE_ADDON_POLLIN		1	// Pollin ADD-ON Board benutzen
	
	#if USE_ADDON_POLLIN
		#define ADDON_POLLIN_IR_RC5	1	// Infrarotempfnger mit RC5-Codierung
	#endif								// Einstellungen in infrared/rc5.h beachten

	//-----------------------------------------------------------------------
	// LCD einbinden (hat bei USE_NETIO immer Vorrang - siehe ab Zeile 414)
	//-----------------------------------------------------------------------

	#define USE_LCD				1	// LCD benutzen (alle Modi)
									// Der serielle Modus an PORT C wird benutzt wenn keine
									// der nchsten vier Einstellungen gesetzt wird
	#if USE_LCD
		#define LCD_4Bit		0	// LCD im 4Bit Mode an PORTC (1) oder seriell (0)
		#define LCD_EXT			0	// LCD im 4Bit Mode am Ext.Port des AVR-NET-IO
		#define LCD_EXT_POLLIN	0	// wie LCD_EXT mit Pollin Verdrahtung (Handbuch S.16)
		#define	LCD_EXT_SERIELL	1	// LCD im seriellen Modus am EXT-Port des AVR-NET-IO

		#define LCD_USER_CHARS	1	// sollen benutzerdefinierte Zeichen benutzt werden?
		#define LCD_ONE_LINE	0	// 0 = mehrzeiliges Display / 1 = 1-zeiliges Display
		#define	LCD_4x20		0	// 0 = alles kleiner als 4x20 / 1 = 4x20
		
		#define USE_LCD_UDP		0	// soll das Display ber UDP angesteuert werden?

		#define	LCD_BACKLIGHT	1	// LCD hat Hintergrundbeleuchtung (4Bit,EXT,EXT_POLLIN)

		#if LCD_BACKLIGHT
			#define	LCD_BL_AUTO	1	// 1 = Hintergrundbeleuchtung automatisch abschalten
			#define LCD_BL_TIME	15	// Sekunden nach denen Beleuchtung abgeschaltet wird
		#endif
	#endif

	//-----------------------------------------------------------------------
	// Servo einbinden
	//-----------------------------------------------------------------------

	#define USE_SERVO		0
	
	//-----------------------------------------------------------------------
	// Kamera einbinden
	//-----------------------------------------------------------------------
	// Kamera arbeitet nur mit einem 14,7456Mhz Quarz!
	// In cam.c knnen weitere Parameter eingestellt werde
	// z.B. Licht, Kompression usw.
	
	#define USE_CAM			0
	#define CAM_RESOLUTION	0	// Auflsung der Kamera
	
	// Auflsungen:
	// 0 = 160x120 Pixel krzer (zum testen OK ;-)
	// 1 = 320x240 Pixel ca. 10 Sek. bei einem Mega644
	// 2 = 640x480 Pixel lnger (dauert zu lang!)
	
	//-----------------------------------------------------------------------
	// Emailversand benutzen? Konfiguration des Emailclient in der sendmail.h
	//-----------------------------------------------------------------------

    #define USE_MAIL        0
    
	//-----------------------------------------------------------------------
    // Empfang von Wetterdaten auf der Console (ber HTTP_GET)
	//-----------------------------------------------------------------------

    #define GET_WEATHER     0

	//-----------------------------------------------------------------------
	// Benutze Werte aus EEPROM und Befehle fr EEPROM Werte
	//-----------------------------------------------------------------------
	
	#define USE_EEPVAR		1	// Startwerte / Sensoren-IDs aus EEPROM lesen
	#define USE_EEPVAR_CMD	1	// EEPVAR-Commands auf USART zulassen (0/1)
	
	//-----------------------------------------------------------------------
	// 74HC165 Schieberegister / Porterweiterung fr Eingnge
	//-----------------------------------------------------------------------
	
	#define USE_SR165			0	// 74HC595 Schieberegister benutzen
	
	#if USE_SR165
		#define SR165_SIZE		1	// Anzahl der 74HC165 ICs
		#define SR165_PORTA		0	// Schieberegister an PORTA * / Pins 1+2 dann OUTPUT
		#define SR165_PORTC		0	// Schieberegister an PORTC *
		#define SR165_PORTC_HI	0	// 0 = Pin 0-2 / 1 = Pin 4-6 - nur PORTC
		#define SR165_PORTD		0	// Schieberegister an PORTD * / Kein LCD mglich
	#endif

	// *) entweder SR165_PORTA, SR165_PORTC oder SR165_PORTD !!!
	
	//-----------------------------------------------------------------------
	// 74HC595 Schieberegister / Porterweiterung fr Ausgnge (Software SPI)
	//-----------------------------------------------------------------------
	
	#define USE_SR595			0	// 74HC595 Schieberegister benutzen
	
	#if USE_SR595
		#define SR595_SIZE		1	// Anzahl der 74HC595 ICs
		#define SR595_WITH_OE	1	// Output Enable benutzen (empfohlen)
		#define SR595_PORTA		0	// Schieberegister an PORTA * / Pins 0-3 dann OUTPUT
		#define SR595_PORTC		0	// Schieberegister an PORTC *
		#define SR595_PORTC_HI	0	// 0 = Pin 0-3 / 1 = Pin 4-7 - nur PORTC
		#define SR595_PORTD		0	// Schieberegister an PORTD * / Kein LCD mglich
	#endif
	
	// *) entweder SR595_PORTA, SR595_PORTC oder SR595_PORTD !!!

	//-----------------------------------------------------------------------
	// Sechs zustzliche Ein- oder Ausgabeports an PORTD (EXT am AVR-NET-IO)
	//-----------------------------------------------------------------------

	#define	USE_PORTD_6IN		0	// 6 zusatzliche Eingnge an PORTD *
	#define USE_PORTD_6OUT		0	// 6 zusatzliche Ausgnge an PORTD *
	
	// *) entweder PORTD_6IN 1 oder PORTD_6OUT 1 oder beide 0 !!!
	
	//-----------------------------------------------------------------------
	// Kopplung mit 2. AVR-NET-IO ber UDP
	//-----------------------------------------------------------------------

	#define USE_UDPSEND			1
	
	#if USE_UDPSEND
		#define UDPSEND_RPORT	10011				// Ein Empfangs-Port muss beim Senden angegeben Werden.
		#define UDPSEND_SPORT	0					// Senden ( Ein ) wenn der Port != 0 ist
		#define UDPSEND_REFRESH	60					// Intervall in Sekunden fr Senden der Daten
		#define UDPSEND_IP		IP(192,168,0,91)	// IP des zweiten Boards (Intranet)
	#endif
	
	//-----------------------------------------------------------------------
	// TFTP-Bootloader
	//-----------------------------------------------------------------------

	#define	USE_TFTPBOOT	1	// 0 = Nur Hardware Reset / 1 = REBOOT Command verfgbar

	#if USE_TFTPBOOT
		#define	TFTP_CMD	1	// 0 spart etwas Platz, aber keine Einstellungen auf der Console mglich
	#endif

	//-----------------------------------------------------------------------
	// 1-Wire Bus fr Temperatur-Sensoren
	//-----------------------------------------------------------------------
	
	#define USE_OW				1	// 1-Wire bus einbinden
	#define OW_MAXSENSORS		8	// max. Anzahl der 1-wire sensoren (DS18x20)
	#define OW_ONE_BUS			1	// nur ein fest eingestellter 1-wire bus; keine Auswahl

	#define OW_EXTERN_POWERED	1	// falls der 1-Wire Bus eine externe Versorgungsspannung hat
	#define OW_ROMCODE_SIZE 	8	// rom-code DS18x20 size including CRC
	
	#define OW_START			150	// Sensorstartwert 15,0 Grad in ow_array
	#define OW_MINMAX_START		0	// Min- / Maxwerte Startwert 0,0 Grad
	#define	OW_MINMAX			1	// Min- / Maxwerte speichern

	#define OW_PORTA			1	// 1-Wire Bus an PORTA7 *
	#define OW_PORTD			0	// 1-Wire Bus an PORTD3 und serielles LCD *
	
	//	*) entweder OW_PORTA 1 oder OW_PORTD 1
		
	#if USE_OW
		#undef	USE_EEPVAR
		#define USE_EEPVAR		1	// mu 1 sein da ROM-IDs im EEPROM gespeichert werden
	#endif

	//=======================================================================
	// Speicherstellen im EEPROM
	//=======================================================================

	#define IP_EEPROM_STORE			30	// IP Adresse *
	#define NETMASK_EEPROM_STORE	34	// Netzmaske *
	#define ROUTER_IP_EEPROM_STORE	38	// IP Gateway *
	#define DNS_IP_EEPROM_STORE		42	// IP Adresse DNS *
	//#define ??_IP_EEPROM_STORE	46	// 4 Byte **
	#define NTP_IP_EEPROM_STORE 	50	// IP Adresse NTP-Server *
	#define MAC_EEPROM_STORE		54	// 6 Byte
	#define WOL_MAC_EEPROM_STORE	60	// Wake On Lan MAC-Adresse
	#define WOL_BCAST_EEPROM_STORE	66	// Wake On Lan Broadcast-Adresse
	#define SYSLOG_IP_EEPROM_STORE	70	// IP Adresse SYSLOG-Server
	#define UDPSEND_IP_EEPROM_STORE	74	// IP des 2. Boards im EEPROM **
	#define UDPSEND_RP_EEPROM_STORE	78	// Receive-Port im EEPROM **
	#define UDPSEND_SP_EEPROM_STORE	80	// Send-Port im EEPROM **
	
	#define TFTP_IP_EEPROM_STORE	108	// IP des TFTP-Server
	#define TFTP_FILE_EEPROM_STORE	112	// Dateiname Hex-Datei (16 Byte)
	#define DEFPC_EEPROM_STORE		128	// Starteinstellung PORTC
	#define DEFPD_EEPROM_STORE		129	// Starteinstellung PORTD

	#define SV_EEPROM_STORE			160	// var_array Speicherung im EEPROM
	#define OW_EEPROM_STORE 		640	// 1-Wire Sensoren

	//   *)	Werte in diesen Stellen werden bei DHCP bergangen.
	//  **)	derzeit ungenutzt

	//=======================================================================
	// Voreinstellungen AVR-NET-IO
	//=======================================================================

	#if USE_NETIO	
		#undef	MYIP
		#define MYIP		IP(192,168,0,90)	// Pollin Default IP Adresse
		#undef	ROUTER_IP
		#define ROUTER_IP	IP(192,168,0,254)	// Pollin Default Gateway
		#undef	NETMASK
		#define NETMASK		IP(255,255,255,0)	// Netzwerkmaske  

		//-------------------------------------------------------------------
		// ab hier nur ndern wenn man wei was man macht!
		//-------------------------------------------------------------------
		
		#undef	MAX_VAR_ARRAY
		#define MAX_VAR_ARRAY	28	// USE_NETIO braucht ein greres Array

		#undef	USE_ENC28J60
		#define USE_ENC28J60	1	// Nie ndern bei Pollin AVR-NET-IO
		#undef	USE_RTL8019
		#define USE_RTL8019		0	// Nie ndern bei Pollin AVR-NET-IO
		#undef	EXTCLOCK
		#define EXTCLOCK		0	// Nie ndern bei Pollin AVR-NET-IO

		#undef	ADC_START			// AVR-NET-IO hat 4 ADC-Kanle (A4-A7)
		#define	ADC_START		4	// 0 => A0-A3 auch als analoge Eingnge

		#undef	OUTA
		#define OUTA	 		0x00	// PORTA sind Eingnge
		#undef	OUTC
		#define OUTC 			0xFF	// PORTC sind Ausgnge
		#undef	OUTD
		#define OUTD 			0xFC	// PORTD 6x Ausgang + 2 Pins fr RS232

		#if USE_SR165
			#if SR165_PORTA
				#undef	OUTA
				#define OUTA	0x06	// wenn Schieberegister auf PORTA
			#endif						// Pins 1+2 als Ausgang
		#endif

		#if USE_SR595
			#if SR595_PORTA
				#undef	OUTA
				#define OUTA	0x0F	// wenn Schieberegister auf PORTA
			#endif						// Pins 0-3 als Ausgang
		#endif

		#if LCD_EXT					// LCD am Ext.Port
			#undef	LCD_4Bit
			#define LCD_4Bit		1	// LCD_EXT bentigt LCD_4Bit
			#undef	USE_SERVO
			#define	USE_SERVO		0	// LCD hat Vorrang - kein Servo an PORTD
			#undef	USE_PORTD_6IN
			#define	USE_PORTD_6IN	0	// LCD hat Vorrang - keine Eingnge an PORTD
			#undef	USE_PORTD_6OUT
			#define	USE_PORTD_6OUT	0	// LCD hat Vorrang - keine Ausgnge an PORTD
			#if USE_SR165
			#if SR165_PORTD
				#undef	USE_SR165
				#define	USE_SR165	0	// LCD hat Vorrang - kein SR165 an PORTD
			#endif	//SR165_PORTD
			#endif	//USE_SR165
			#if USE_SR595
			#if SR595_PORTD
				#undef	USE_SR595
				#define	USE_SR595	0	// LCD hat Vorrang - kein SR595 an PORTD
			#endif	//SR595_PORTD
			#endif	//USE_SR595
			#if ADDON_POLLIN_IR_RC5		// LCD hat Vorrang - kein Infrarot an PB0
				#undef	ADDON_POLLIN_IR_RC5
				#define	ADDON_POLLIN_IR_RC5 0
			#endif
		#endif
	
		#if LCD_EXT_POLLIN			// LCD am Ext.Port mit Pollin Verdrahtung
			#undef	LCD_4Bit
			#define LCD_4Bit		1	// LCD_EXT_POLLIN bentigt LCD_4Bit
			#undef	LCD_EXT
			#define LCD_EXT			1	// LCD_EXT_POLLIN bentigt LCD_EXT
			#undef	USE_SERVO
			#define	USE_SERVO		0	// LCD hat Vorrang - kein Servo an PORTD
			#undef	USE_PORTD_6IN
			#define	USE_PORTD_6IN	0	// LCD hat Vorrang - keine Eingnge an PORTD
			#undef	USE_PORTD_6OUT
			#define	USE_PORTD_6OUT	0	// LCD hat Vorrang - keine Ausgnge an PORTD
			#if USE_SR165
			#if SR165_PORTD
				#undef	USE_SR165
				#define	USE_SR165	0	// LCD hat Vorrang - kein SR165 an PORTD
			#endif	//SR165_PORTD
			#endif	//USE_SR165
			#if USE_SR595
			#if SR595_PORTD
				#undef	USE_SR595
				#define	USE_SR595	0	// LCD hat Vorrang - kein SR595 an PORTD
			#endif	//SR595_PORTD
			#endif	//USE_SR595
		#endif

		#if USE_PORTD_6IN
			#undef	OUTD
			#define OUTD 			0x00	// PORTD 6x Eingang + 2 Pins fr RS232
		#endif

		//===================================================================
		// AVR-NET-IO mit ATmega32 - nur ndern wenn man wei was man macht!
		//===================================================================

		#if USE_NETIO_32	// Bildet Funktionsweise des Standard AVR-NET-IO ab
			#undef	USE_ADC
			#define USE_ADC			1
			#undef	ADC_START			// AVR-NET-IO hat 4 ADC-Kanle (A4-A7)
			#define	ADC_START		4	// 0 => A0-A3 auch als analoge Eingnge
			#undef	USE_DHCP
			#define USE_DHCP		0	// passt evtl. zustzlich zum Standard in ATMega32
			#undef	USE_NTP
			#define USE_NTP			0
			#undef	USE_DNS
			#define USE_DNS			0
			#undef	USE_SYSLOG
			#define USE_SYSLOG		0
			#undef	USE_WOL
			#define USE_WOL			0
			#undef	USE_MAIL
			#define USE_MAIL		0
			#undef	GET_WEATHER
			#define GET_WEATHER		0
			#undef	USE_SERVO
			#define USE_SERVO		0
			#undef	USE_CAM
			#define USE_CAM			0
			#undef	USE_OW
			#define USE_OW			0
			#undef	USE_UDPSEND
			#define	USE_UDPSEND		0
			#undef	USE_EEPVAR
			#define USE_EEPVAR		0
			#undef	USE_EEPVAR_CMD
			#define USE_EEPVAR_CMD	0
			#undef	USE_NETCMDS
			#define USE_NETCMDS		0
			#undef	USE_SR165
			#define	USE_SR165		0
			#undef	USE_SR595
			#define	USE_SR595		0
			#undef	USE_PORTD_6IN
			#define	USE_PORTD_6IN	0
			#undef	USE_PORTD_6OUT
			#define	USE_PORTD_6OUT	0
			#undef	USE_TFTPBOOT
			#define	USE_TFTPBOOT	0
			#undef	TFTP_CMD
			#define	TFTP_CMD		0
			#undef	ADDON_POLLIN_IR_RC5
			#define	ADDON_POLLIN_IR_RC5 0

			#undef	F_CPU
			#define	F_CPU 16000000UL

			#undef MAC_EEPROM_STORE
			#define MAC_EEPROM_STORE		1006	// Pollin Default bei ATmega32
			#undef ROUTER_IP_EEPROM_STORE
			#define ROUTER_IP_EEPROM_STORE	1012	// Dadurch werden die Einstellungen
			#undef NETMASK_EEPROM_STORE				// die in der Originalsoftware
			#define NETMASK_EEPROM_STORE	1016	// gemacht wurden bernommen, wenn
			#undef IP_EEPROM_STORE					// EEPROM beim flashen nicht
			#define IP_EEPROM_STORE			1020	// gelscht wurde.
		#endif

		#define URVersion       "1.2.5"		// Ulrich Radigs Original Version
		#define Version         "1.0.19"	// Diese Version
	#endif

	extern unsigned int var_array[MAX_VAR_ARRAY];
	#if USE_UDPSEND
	extern unsigned int va2_array[MAX_VAR_ARRAY];
	#endif
	
#endif //_CONFIG_H
