/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        angepasst von Shortie
 known Problems: none
 Version:        31.01.2010
 Description:    Webserver uvm.

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/

#include "config.h"
#include "usart.h"
#include "networkcard/enc28j60.h"
#include "networkcard/rtl8019.h"
#include "stack.h"
#include "timer.h"
#include "wol.h"
#include "httpd.h"
#include "cmd.h"
#include "telnetd.h"
#include "ntp.h"
#include "base64.h"
#include "http_get.h"
#include "lcd.h"
#include "udp_lcd.h"
#include "analog.h"
#include "camera/cam.h"
#include "camera/servo.h"
#include "sendmail.h"
#include "dhcpc.h"
#include "dnsc.h"
#include "syslog/syslog.h"
#include "udp_send/udp_send.h"
#include "sr74hc165/sr74hc165.h"
#include "sr74hc595/sr74hc595.h"
#include <avr/io.h>
#include <avr/eeprom.h>
#include <util/delay.h>

#if USE_OW
#include <stdlib.h>
#include "messung.h"
#include "1-wire/ds18x20.h"

uint8_t ow_auslesen	= 0;	// 0 -> Sensoren nicht auslesen
uint8_t minmax		= 1;	// 1 -> Zurcksetzen der Min/Max-Werte 1x/Tag
#if OW_MINMAX
int16_t ow_array[OW_MAXSENSORS*3];
#else
int16_t ow_array[OW_MAXSENSORS];
#endif //OW_MINMAX
#endif

#if LCD_BL_AUTO
uint8_t bl_automatik = 0;	// 0 -> Backlight Automatik aus
#endif

#if USE_DHCP
unsigned int dhcpok;
#endif
#if USE_NETIO

#if ADDON_POLLIN_IR_RC5
	#include "infrared/rc5.h"
#endif

//-----------------------------------------------------
// debounce Funktion um Taster zu entprellen
//-----------------------------------------------------

inline uint8_t debounce(volatile uint8_t *port, uint8_t pin)
{
	if ( ! (*port & (1 << pin)) )
	{
		/* Pin wurde auf Masse gezogen, 100ms warten   */
		long_delay(100);  
		if ( *port & (1 << pin) )
        {
			/* Anwender Zeit zum Loslassen des Tasters geben */
			long_delay(50);
			return 1;
		}
	}
	return 0;
}

//----------------------------------------------------------------------------
// Variablen-Array / Variablen-Array2 bei USE_UDPSEND
//----------------------------------------------------------------------------
// zum Abspeichern verschiedener Werte und zum Einfgen in die Webseite 
// als %VA@00 bis %VA@xx
//
// VA0-7	-> PORT A0-7
// VA8		-> PORT C
// VA9		-> Port D
// VA10		-> Sonderfunktionen (derzeit: Bit 1: ow_auslesen, Bit 2: bl_automatik)
// VA11		-> Tasterdefinition PORT C
// VA12		-> Tastdauer von VA11
// VA13-26	-> Reserve (14 Werte)
// VA27		-> Counter Webseite

unsigned int var_array[MAX_VAR_ARRAY] = {0,0,0,0,0,0,0,0,0,0, 3, 0,500, 0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0};
	#if USE_UDPSEND
	unsigned int va2_array[MAX_VAR_ARRAY] = {0,0,0,0,0,0,0,0,0,0, 0, 0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0};
	#endif	//USE_UDPSEND
#else	//USE_NETIO
unsigned int var_array[MAX_VAR_ARRAY] = {10,50,30,0,0,0,0,0,0,0};
	#if USE_UDPSEND
	unsigned int va2_array[MAX_VAR_ARRAY] = {0,0,0,0,0,0,0,0,0,0};
	#endif //USE_UDPSEND
#endif //USE_NETIO

//----------------------------------------------------------------------------
// Hier startet das Hauptprogramm
//----------------------------------------------------------------------------

int main(void)
{  	
    unsigned long a;
	
	//Konfiguration der Ausgnge bzw. Eingnge
	//definition erfolgt in der config.h
	DDRA = OUTA;
	DDRC = OUTC;
	DDRD = OUTD;
	
	#if USE_NETIO
	PORTC = 0;
	// Pullups einschalten PortA
	#if !SR595_PORTA
	PORTA |= (1 << PORTA0) | (1 << PORTA1) | (1 << PORTA2) | (1 << PORTA3);
	#endif
	
	#if !USE_ADC
	PORTA |= (1 << PORTA4) | (1 << PORTA5) | (1 << PORTA6);
	#endif
	
	#if !OW_PORTA
	#if !USE_ADC
		PORTA |= (1 << PORTA7);
	#endif
	#endif
	
	DDRB &= ~( 1 << DDB1 ); // Pin B1 (Pollin Prog.-Jumper) als Eingang definieren
	PORTB |= (1 << PORTB1); // Pullup einschalten PortB1
	#endif //USE_NETIO
	
	#if USE_PORTD_6IN	// Pullup einschalten Ports D2-D7
	PORTD |= (1 << PORTD2) | (1 << PORTD3) | (1 << PORTD4) | (1 << PORTD5) | (1 << PORTD6) | (1 << PORTD7);
	#endif
	
	#if USE_SR595
	sr74hc595_init();
	#endif

	eepvar_init();	// Grundeinstellungen einlesen

	#if USE_SERVO
		servo_init ();
	#endif //USE_SERVO
	
    usart_init(BAUDRATE); // setup the UART
	
	#if USE_ADC
		ADC_Init();
	#endif
	
	usart_write("\r\nSystem Ready\r\n");
    usart_write("Compiliert am "__DATE__" um "__TIME__"\r\n");
    usart_write("Compiliert mit GCC Version "__VERSION__"\r\n\r\n");

	for(a=0;a<1000000;a++){asm("nop");};

	//Applikationen starten
	stack_init();
	
	#if USE_HTTPD
	httpd_init();
	#endif
	
	#if USE_TELNETD
	telnetd_init();
	#endif
	
	//LCD initialisieren
	#if USE_LCD
		#if USE_LCD_UDP
		udp_lcd_init();
		#endif
		lcd_init();
		lcd_clear();
		
		#if LCD_BACKLIGHT
		#if LCD_BL_AUTO
			lcd_bl_timer = LCD_BL_TIME;
			lcd_backlight = 1;
		#endif	//LCD_BL_AUTO
		#endif	//LCD_BACKLIGHT

		//Spielerrei mit einem LCD
		lcd_print(0,1,"System Booting");
		#if LCD_USER_CHARS
		lcd_print(1,3,"I %c my AVR",3);
		#endif
	#endif

	//Ethernetcard Interrupt enable
	ETH_INT_ENABLE;
	
	//Globale Interrupts einschalten
	sei(); 
	
	#if USE_CAM
		#if USE_LCD
			lcd_print(1,0,"CAMERA INIT");
		#endif //USE_LCD
		for(a=0;a<2000000;a++){asm("nop");};
		cam_init();
		max_bytes = cam_picture_store(CAM_RESOLUTION);
		#if USE_LCD
			lcd_print(1,0,"CAMERA READY");
		#endif //USE_LCD
	#endif //USE_CAM

    #if USE_DHCP
    dhcp_init();
    if ( dhcp() == 0)
    {
        save_ip_addresses();
		dhcpok = 1;
    }
    else
    {
        usart_write("DHCP fail\r\n\r\n");
        read_ip_addresses(); //get from EEPROM
		dhcpok = 0;
    }
    #endif //USE_DHCP
	
    usart_write("IP   %1i.%1i.%1i.%1i"    , myip[0]     , myip[1]     , myip[2]     , myip[3]);
	#if USE_DHCP
	if(dhcpok == 1) {
		usart_write(" (DHCP)\r\n");
	} else {
		usart_write("\r\n");
	}
	#endif //USE_DHCP
    usart_write("MASK %1i.%1i.%1i.%1i\r\n", netmask[0]  , netmask[1]  , netmask[2]  , netmask[3]);
    usart_write("GW   %1i.%1i.%1i.%1i\r\n", router_ip[0], router_ip[1], router_ip[2], router_ip[3]);

    #if USE_DNS
    usart_write("DNS  %1i.%1i.%1i.%1i\r\n", dns_server_ip[0], dns_server_ip[1], dns_server_ip[2], dns_server_ip[3]);
    #endif //USE_DNS
    
	#if USE_NTP
	  #if !USE_DHCP
        #if USE_DNS
        dns_init();
        if ( dns_resolve("1.de.pool.ntp.org") == 0) //resolve NTP server
        {
          for (unsigned char count = 0; count<4; count++)
          {
            eeprom_busy_wait ();
            eeprom_write_byte((unsigned char *)(NTP_IP_EEPROM_STORE + count),dns_resolved_ip[count]);
          }
        }
        else
        {
            usart_write("DNS Err.\r\n");
        }
        #endif //USE_DNS
	  #endif //USE_DHCP
    
   ntp_init(); 
   
   usart_write("NTP  %1i.%1i.%1i.%1i\r\n", ntp_server_ip[0], ntp_server_ip[1], ntp_server_ip[2], ntp_server_ip[3]);
   
   for(a=0;a<1000000;a++){asm("nop");};
   
   ntp_request();
   
    if ( ntp() != 0 )
	{
	  usart_write("NTP Err.\r\n");
	}
    else
	{
	  command_time();
	}
	#endif //USE_NTP
	
	#if USE_WOL
        wol_init();
	#endif //USE_WOL
    
    #if USE_MAIL
        mail_client_init();
	#endif //USE_MAIL  

	#if USE_OW
		uint8_t i;
		for (i=0;i<OW_MAXSENSORS;i++){
			ow_array[i]=OW_START;
		}
		#if OW_MINMAX
		for (i=OW_MAXSENSORS;i<OW_MAXSENSORS*3;i++){
			ow_array[i]=OW_MINMAX_START;
		}
		#endif //OW_MINMAX
		messung_init();
    #endif

	#if ADDON_POLLIN_IR_RC5
		rc5_init();
	#endif

	#if USE_UDPSEND
		udp_send_init();
	#endif

	#if USE_SYSLOG
		usart_write("SYSLOG  %1i.%1i.%1i.%1i\r\n", syslog_server_ip[0], syslog_server_ip[1], syslog_server_ip[2], syslog_server_ip[3]);
		syslog_send(5,0,"system: AVR-NET-IO is running");
	#endif
	
	#if USE_LCD
		lcd_clear();
		lcd_print(0,0,"System ready   ");
	#endif

//---------------------------------------------------------------------
// Hauptschleife
//---------------------------------------------------------------------

	while(1)
	{
		unsigned char hh = (time/3600)%24;
        unsigned char mm = (time/60)%60;
        unsigned char ss = time %60;
	
		//-----------------------------------------------------
		// Variable mit Portwerten fllen
		//-----------------------------------------------------

		#if USE_NETIO		
		var_array[0] = (PINA&0b00000001);
		var_array[1] = (PINA&0b00000010)>>1;
		var_array[2] = (PINA&0b00000100)>>2;
		var_array[3] = (PINA&0b00001000)>>3;
		
		// ADC-Werte werden in analog.c in die Variablen geschrieben
		// hier werden bei USE_ADC 0 die Werte geschrieben.
		#if !USE_ADC
		var_array[4] = (PINA&0b00010000)>>4;
		var_array[5] = (PINA&0b00100000)>>5;
		var_array[6] = (PINA&0b01000000)>>6;
		
		#if OW_PORTA
		var_array[7] = 0;
		#else
		var_array[7] = (PINA&0b10000000)>>7;
		#endif	//OW_PORTA
		#endif	//USE_ADC
		
		var_array[8] = PORTC;
		var_array[9] = PORTD;	// (sinnlos bei LCD oder Servo an PORTD)
		#endif	//USE_NETIO

		//-----------------------------------------------------

		#if USE_ADC
		ANALOG_ON;
		#endif
		
	    eth_get_data();
		
        //Terminalcommandos auswerten
		if (usart_status.usart_ready){
            usart_write("\r\n");
			if(extract_cmd(&usart_rx_buffer[0]))
			{
				usart_write("Ready\r\n\r\n");
			}
			else
			{
				usart_write("ERROR\r\n\r\n");
			}
			usart_status.usart_ready =0;
		}
		
        //Wetterdaten empfangen (Testphase)
        #if GET_WEATHER
        http_request ();
        #endif
        
        //Empfang von Zeitinformationen
		#if USE_NTP
		if(!ntp_resync)
		{
			ntp_resync = NTP_REFRESH;
			ntp_request();
		}
		#endif //USE_NTP
	
        //Versand von E-Mails
        #if USE_MAIL
        if (mail_enable == 1)
        {
            mail_enable = 0;
            mail_send();
        }
        #endif //USE_MAIL
        
        //Rechner im Netzwerk aufwecken
        #if USE_WOL
        if (wol_enable == 1)
        {
            wol_enable = 0;
            wol_request();
        }
        #endif //USE_WOL
        
        #if USE_DHCP
		if(dhcpok == 1) {		// nur ausfuehren wenn DHCP beim Start verfgbar war
			if ( dhcp() != 0)	//check for lease timeout
			{
				usart_write("dhcp lease renewal failed\r\n");
				RESET();
			}
		}
		#endif //USE_DHCP
  
		#if USE_TELNETD
		telnetd_send_data();	//USART Daten fr Telnetanwendung?
		#endif
        
        if(ping.result)
        {
            usart_write("Get PONG: %i.%i.%i.%i\r\n",ping.ip1[0],ping.ip1[1],ping.ip1[2],ping.ip1[3]); 
            ping.result = 0;
        }

		#if (UDPSEND_SPORT!=0)
		if(!udp_send_timer)
		{
			udp_send(UDPSEND_IP);
			udp_send_timer = UDPSEND_REFRESH;
		}
		#endif //UDPSEND_SPORT

		#if LCD_BL_AUTO
		bl_automatik = ((var_array[10]&0b00000010)>>1);	// Bit 2 von var_array[10] auslesen
		if (bl_automatik == 1)
		{
			if((!lcd_bl_timer) && (lcd_backlight))	// Wenn lcd_bl_timer = 0 und lcd_backlight = 1
			{
				#if LCD_EXT
				LCD_Port2_Write &= ~(1<<LCD_LIGHT_PIN);	// LCD-Backlight ausschalten
				#endif
				lcd_backlight = 0;
			}
		}
		#endif	//LCD_BL_AUTO
		
		//-------------------------------------------------------------------
		// Infrarotempfnger auslesen und RC5 Code interpretieren
		//-------------------------------------------------------------------
		
		#if ADDON_POLLIN_IR_RC5
		cli();
		int fbc = rc5_data;	// read two bytes from interrupt !
		rc5_data = 0;
		sei();
		
		if( fbc ){
			if ( fbc != lastRC5data ) {
				// neuer Tastendruck
				lastRC5data = fbc;	// letzten Tastendruck speichern

				rc5_device = (fbc >> 6 & 0x1F);
				rc5_keycode = (fbc & 0x3F) | (~fbc >> 7 & 0x40);

				RC5_DEBUG("\r\nRC5 %i",( fbc >> 11 & 1));	// Toggle Bit
				RC5_DEBUG(" %i", rc5_device);				// Device address
				RC5_DEBUG(" %i -",rc5_keycode);				// Key Code
				
				// RC5_DEBUG kann in infrared/rc5.h aktiviert werden um
				// um weitere Tastencodes einer RC5-Fernbedienung zu erfassen
				
				//-----------------------------------------------------------
				// Schaltvorgnge ber Fernbedienung
				//-----------------------------------------------------------

//				if (rc5_device == RC5DEV_ID)		// Nur Codes fr angebene Device Adresse verarbeiten
//				{									// Wenn mehrere Gerte RC5 Codes benutzen
					if (rc5_keycode == RC5KEY_0)	// Wenn FB-Taste 0 gedrckt
						PORTC = 0x00;				// schalte alle Ausgnge an PORTC aus

					if (( rc5_keycode >= RC5KEY_1 ) && (rc5_keycode <= RC5KEY_8))	// Wenn FB-Taste 1 - 8
						PORTC = PINC ^ (1 << (rc5_keycode - 1));					// schalte Ausgang PC0 - PC7

					if (rc5_keycode == RC5KEY_9)	// Wenn FB-Taste 9 gedrckt
						PORTC = 0xFF;				// schalte alle Ausgnge ein

					#if LCD_BACKLIGHT
					if (rc5_keycode == RC5KEY_VUP)	lcd_backlight = 1;	// FB-Taste VOL+
					if (rc5_keycode == RC5KEY_VDN)	lcd_backlight = 0;	// FB-Taste VOL-
					#endif
//				}
			} else {
				RC5_DEBUG("\r\nTaste %i wird gehalten",rc5_keycode);
			}
		}
		#endif

		//-----------------------------------------------------
		// 1-Wire Bus abfragen und Temperaturen auslesen
		//-----------------------------------------------------
		
		#if USE_OW
		
		ow_auslesen = (var_array[10] & 0x01);	// Bit 1 von var_array[10] auslesen
		
		if ((ss%10 == 8) && (ow_auslesen != 0)) 
		{
			#if OW_EXTERN_POWERED
			start_OW();
			#endif
			lese_Temperatur();	// Temperatursensoren lesen

			#if OW_MINMAX
			for (i=0;i<OW_MAXSENSORS;i++){
				if ((( hh == 00 )&&( mm == 00 )) || (minmax == 1)) {
					// Zurcksetzen der Min/Max_Werte 1x/Tag
					// auf die gerade aktuellen Temperaturen
					ow_array[i+OW_MAXSENSORS] = ow_array[i];
					ow_array[i+OW_MAXSENSORS*2] = ow_array[i];
				} else {
					// Abgleich der Temp. mit den gespeicherten Min/Max-Werten
					if (ow_array[i]  < ow_array[i+OW_MAXSENSORS])
					ow_array[i+OW_MAXSENSORS] = ow_array[i];
					if (ow_array[i]  > ow_array[i+OW_MAXSENSORS*2])
					ow_array[i+OW_MAXSENSORS*2] = ow_array[i];
				}
			}
			minmax = 0;
			#endif //OW_MINMAX
			
			// Beispiel fr unmittelbare Temperaturspeicherung in Variable:
			// var_array[25] = ow_array[0];
			// var_array[26] = ow_array[1];
			
		}
		#endif	//USE_OW

		//-----------------------------------------------------
		// Taster an PB1 abfragen und LCD-Backlight schalten
		//-----------------------------------------------------

		#if LCD_BACKLIGHT
		if (debounce(&PINB, PB1))	// Falls Taster an PIN PB1 gedrueckt..
		{
			#if LCD_EXT
			LCD_Port2_Write = LCD_Port2_Read ^ ( 1 << LCD_LIGHT_PIN );	// LCD-Backlight an- bzw. ausschalten
			#else
			if(lcd_backlight == 1) lcd_backlight = 0;
			else lcd_backlight = 1;
			#endif	//LCD_EXT
			
			#if LCD_BL_AUTO
			lcd_bl_timer = LCD_BL_TIME;
			lcd_backlight = 1;
			#endif	//LCD_BL_AUTO
		}
		#endif	//LCD_BACKLIGHT

//---------------------------------------------------------------------
// ab hier eigenen Code wie Ausgaben oder weitere Verarbeitung der 
// Werte einfgen
//---------------------------------------------------------------------

		#if USE_LCD
		lcd_print(2,0,"%2i:%2i:%2i",hh,mm,ss);
/*
		// Demo der 8 benutzerdefinierten Zeichen
		#if LCD_USER_CHARS
		uint8_t udc;
		for (udc=0;udc<8;udc++) {
			lcd_print(3,udc,"%c",udc);	// Die 8 benutzerdefinierten Zeichen
		}								// in Zeile 4 des Displays anzeigen
		#endif
*/
		#if USE_OW
		char zeile[20];
		lcd_print(0,0,"Aussen ");
		dtostrf(ow_array[0] / 10.0,5,1,zeile);
		lcd_print_str(zeile);
		lcd_print_str("C");
		lcd_print(1,0,"Innen  ");
		dtostrf(ow_array[1] / 10.0,5,1,zeile);
		lcd_print_str(zeile);
		lcd_print_str("C");
		
		// Beispiel fr Temperaturwertspeicherung in Variable:
		//if(ss%10 == 1) // bei Sekunde 1,11,21,31,41,51
		if(ss == 11) // bei Sekunde 11
		{
		var_array[25] = ow_array[0];
		var_array[26] = ow_array[1];
		}
		
		#if USE_SR165
		sr165_read();
		for(i=0;i<SR165_SIZE;i++)
		{
			lcd_print(3,i*4,"%i",sr165val[i]);	//gebe array-inhalt auf LCD Zeile 4 aus
		}
		#endif	//USE_SR165
		
		#endif	//USE_OW
		#endif	//USE_LCD
    }//while (1)
//---------------------------------------------------------------------
return(0);
}

